@@ -27,10 +27,12 @@ import dbus.mainloop.glib
 import subprocess
 import gobject
 import os
+import shutil
 import re
 import tempfile
 import glob
 import sys
+import datetime
 
 ##                ##
 ##Common Variables##
@@ -77,9 +79,19 @@ UP_FILENAMES =  [ 'upimg.bin',
 ##                ##
 ##Common Functions##
 ##                ##
+
+def black_tree(action, blacklist, src, dst='', base=None):
+    """Recursively ACTIONs files from src to dest only
+       when they don't match the blacklist outlined in blacklist"""
+    return _tree(action, blacklist, src, dst, base, False)
+    
 def white_tree(action, whitelist, src, dst='', base=None):
     """Recursively ACTIONs files from src to dest only
        when they match the whitelist outlined in whitelist"""
+    return _tree(action, whitelist, src, dst, base, True)
+    
+def _tree(action, list, src, dst, base, white):
+    """Helper function for tree calls"""
     from distutils.file_util import copy_file
 
     if base is None:
@@ -107,15 +119,15 @@ def white_tree(action, whitelist, src, dst='', base=None):
         elif os.path.isdir(src_name):
             if action == "copy":
                 outputs.extend(
-                    white_tree(action, whitelist, src_name, dst_name, base))
+                    _tree(action, list, src_name, dst_name, base, white))
             elif action == "size":
                 #add the directory we're in
                 outputs += os.path.getsize(src_name)
                 #add the files in that directory
-                outputs += white_tree(action, whitelist, src_name, dst_name, base)
+                outputs += _tree(action, list, src_name, dst_name, base, white)
 
-        #only copy the file if it matches the whitelist
-        elif whitelist.search(end):
+        #only copy the file if it matches the list / color
+        elif (white and list.search(end)) or not (white or list.search(end)):
             if action == "copy":
                 if not os.path.isdir(dst):
                     os.makedirs(dst)
@@ -135,20 +147,20 @@ def check_vendor():
             vendor = rfd.readline().split()[0].lower()
     else:
         vendor = ''
-    return (vendor == 'dell')
+    return ((vendor == 'dell') or (vendor == 'alienware'))
 
-def check_version():
+def check_version(package='dell-recovery'):
     """Queries the package management system for the current tool version"""
     try:
         import apt.cache
         cache = apt.cache.Cache()
-        if cache['dell-recovery'].is_installed:
-            return cache['dell-recovery'].installed.version
+        if cache[package].is_installed:
+            return cache[package].installed.version
     except Exception, msg:
-        print >> sys.stderr, "Error checking dell-recovery version: %s" % msg
+        print >> sys.stderr, "Error checking %s version: %s" % (package, msg)
         return "unknown"
 
-def process_conf_file(original, new, uuid, rp_number, dual_seed='', ako=''):
+def process_conf_file(original, new, uuid, rp_number, ako='', recovery_text=''):
     """Replaces all instances of a partition, OS, and extra in a conf type file
        Generally used for things that need to touch grub"""
     if not os.path.isdir(os.path.split(new)[0]):
@@ -181,6 +193,8 @@ def process_conf_file(original, new, uuid, rp_number, dual_seed='', ako=''):
     with open(original, "r") as base:
         with open(new, 'w') as output:
             for line in base.readlines():
+                if "#RECOVERY_TEXT#" in line:
+                    line = line.replace("#RECOVERY_TEXT#", recovery_text)
                 if "#UUID#" in line:
                     line = line.replace("#UUID#", uuid)
                 if "#PARTITION#" in line:
@@ -189,8 +203,6 @@ def process_conf_file(original, new, uuid, rp_number, dual_seed='', ako=''):
                     line = line.replace("#OS#", "%s %s" % (release["ID"], release["RELEASE"]))
                 if "#EXTRA#" in line:
                     line = line.replace("#EXTRA#", "%s" % extra_cmdline.strip())
-                if '#DUAL#' in line:
-                    line = line.replace("#DUAL#", "%s"  % dual_seed)
                 output.write(line)
 
 def fetch_output(cmd, data=None):
@@ -203,18 +215,21 @@ def fetch_output(cmd, data=None):
         proc.wait()
     if proc.returncode != 0:
         error = "Command %s failed with stdout/stderr: %s\n%s" % (cmd, out, err)
+        import syslog
         syslog.syslog(error)
         raise RuntimeError, (error)
     return out
 
 def find_supported_ui():
     """Finds potential user interfaces"""
-    desktop = { 'ude'     : 'gnome.desktop',
-                'une-efl' : 'une-efl.desktop',
-                'une'     : 'une.desktop'}
-    name =    { 'ude'     : 'Desktop Edition',
-                'une'     : 'Netbook Edition',
-                'une-efl' : 'Netbook Edition (2D)'}
+    desktop = { 'gnome'             : 'gnome.desktop',
+                'unity-2d'          : 'unity-2d.desktop',
+                'gnome-classic'     : 'gnome-classic.desktop',
+                'gnome-2d'          : 'gnome-2d.desktop'}
+    name =    { 'gnome'             : 'Unity (3D)',
+                'unity-2d'          : 'Unity (2D)',
+                'gnome-classic'     : 'GNOME (Classic 3D)',
+                'gnome-2d'          : 'GNOME (Classic 2D)'}
     for item in desktop:
         if not os.path.exists(os.path.join('/usr/share/xsessions/', desktop[item])):
             name.pop(item)
@@ -500,7 +515,8 @@ def walk_cleanup(directory):
 
 def create_new_uuid(old_initrd_directory, old_casper_directory,
                     new_initrd_directory, new_casper_directory,
-                    new_compression="auto"):
+                    new_compression="auto",
+                    include_bootstrap=False):
     """ Regenerates the UUID contained in a casper initramfs
         Supported compression types:
         * auto (auto detects lzma/gzip)
@@ -513,7 +529,7 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
 
     #Detect the old initramfs stuff
     try:
-        old_initrd_file = glob.glob('%s/initrd*' % old_initrd_directory)[0]
+        old_initrd_files = glob.glob('%s/initrd*' % old_initrd_directory)
     except Exception, msg:
         print str(msg)
         raise dbus.DBusException, ("Missing initrd in image.")
@@ -523,18 +539,28 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
         print str(msg)
         raise dbus.DBusException, ("Missing casper UUID in image.")
 
+    old_initrd_file = ''
+    old_compression = ''
+    old_suffix = ''  
+    for fname in old_initrd_files:
+        if len(fname.split('.')) > 1:
+            old_suffix = fname.split('.')[1]
+        if old_suffix == "lz":
+            old_compression = "lzma"
+            old_initrd_file = fname
+            break
+        elif old_suffix == "gz":
+            old_compression = "gzip"
+            old_initrd_file = fname
+            break
+        else:
+            old_suffix = ''
+
+    if not old_suffix or not old_initrd_file or not old_uuid_file:
+        raise dbus.DBusException, ("Unable to detect valid initrd.")
+
     print "Old initrd: %s" % old_initrd_file
     print "Old uuid file: %s" % old_uuid_file
-
-    old_suffix = ''
-    if len(old_initrd_file.split('.')) > 1:
-        old_suffix = old_initrd_file.split('.')[1]
-
-    old_compression = ''
-    if old_suffix == "lz":
-        old_compression = "lzma"
-    elif old_suffix == "gz":
-        old_compression = "gzip"
     print "Old suffix: %s" % old_suffix
     print "Old compression method: %s" % old_compression
 
@@ -555,6 +581,11 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
         with open(item, "w") as uuid_fd:
             uuid_fd.write(new_uuid)
 
+    #Newer (Ubuntu 11.04+) images may support including the bootstrap in initrd
+    if include_bootstrap:
+        chain0 = subprocess.Popen(['/usr/share/dell/casper/hooks/dell-bootstrap'], env={'DESTDIR': tmpdir, 'INJECT': '1'})
+        chain0.communicate()    
+
     #Detect compression
     new_suffix = ''
     if new_compression == "gzip":
@@ -587,6 +618,109 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
 
     return (old_initrd_file, old_uuid_file)
 
+def create_g2ldr(chroot, rp_mount, install_mount):
+    '''Create a g2ldr compatible image using the install
+       chroot: chroot to launch commands in
+       rp_mount: mountpoint to find partition# and install g2ldr
+       install_mount: mountpoint to find target OS UUID'''
+    bus = dbus.SystemBus()
+    udisk_bus_name = 'org.freedesktop.UDisks'
+    dev_bus_name   = 'org.freedesktop.UDisks.Device'
+    uuid = ''
+    partition = '-1'
+
+    #Find the UUID we are installing to
+    obj = bus.get_object(udisk_bus_name, '/org/freedesktop/UDisks')
+    iface = dbus.Interface(obj, udisk_bus_name)
+    devices = iface.EnumerateDevices()
+    for device in devices:
+        obj = bus.get_object(udisk_bus_name, device)
+        dev = dbus.Interface(obj, 'org.freedesktop.DBus.Properties')
+        mount = dev.Get(dev_bus_name, 'DeviceMountPaths')
+        if mount and install_mount and install_mount in mount:
+            uuid = dev.Get(dev_bus_name, 'IdUuid')
+        elif mount and rp_mount and rp_mount in mount:
+            partition = str(dev.Get(dev_bus_name, 'DeviceMinor'))
+
+    #The file that the Windows BCD will chainload
+    shutil.copy('/usr/lib/grub/i386-pc/g2ldr.mbr', rp_mount)
+    
+    # This BCD configuration will load from the recovery partition if it exists
+    #   a /grub/grub.cfg.  As specified by the active partition flag
+    # If it doesn't, then it will look for /boot/grub/grub.cfg on the OS partition
+    #   as specified by the UUID.
+    process_conf_file('/usr/share/dell/grub/bcd.cfg', \
+                      os.path.join(chroot, 'tmp', 'bcd.cfg'), uuid, partition)
+
+    #Build the Grub2 Core Image
+    build_command = ['grub-mkimage', '-O', 'i386-pc',
+                                     '-c', '/tmp/bcd.cfg', 
+                                     '-o', '/tmp/core.img',
+                                     'biosdisk', 'part_msdos', 'part_gpt', 'ls',
+                                     'fat', 'ntfs', 'ntfscomp', 'search',
+                                     'linux', 'vbe', 'boot', 'minicmd', 
+                                     'cat', 'cpuid', 'chain', 'halt', 'linux16',
+                                     'echo', 'test', 'configfile', 'ext2',
+                                     'keystatus', 'help', 'boot', 'loadenv' ]
+    if chroot == '/target':
+        from ubiquity import install_misc
+        install_misc.chrex(chroot, *build_command)
+    else:
+        from ubiquity import misc
+        misc.execute_root(*build_command)
+
+    with open(os.path.join(rp_mount, 'g2ldr'), 'w') as wfd:
+        for fname in ['/usr/lib/grub/i386-pc/g2hdr.bin', os.path.join(chroot, 'tmp', 'core.img')]:
+            with open(fname) as rfd:
+                wfd.write(rfd.read())
+
+def parse_seed(seed):
+    """Parses a preseed file and returns a set of keys"""
+    keys = {}
+    if os.path.exists(seed):
+        with open(seed, 'r') as rfd:
+            line = rfd.readline()
+            while line:
+                line = line.strip()
+                if line and not line.startswith('#'):
+                    line = line.split()
+                    line.pop(0) # ubiquity or d-i generally
+                    key = line.pop(0)
+                    if '/' in key:
+                        type = line.pop(0)
+                        value = " ".join(line)
+                        keys[key] = value
+                line = rfd.readline()
+    return keys
+
+def write_seed(seed, keys):
+    """Writes out a preseed file with a selected set of keys"""
+    with open(seed, 'w') as wfd:
+        wfd.write("# Dell Recovery configuration preseed\n")
+        wfd.write("# Last updated on %s\n" % datetime.date.today())
+        wfd.write("\n")
+        for item in keys:
+            if keys[item] == 'true' or keys[item] == 'false':
+                type = 'boolean'
+            else:
+                type = 'string'
+            wfd.write(" ubiquity %s %s %s\n" % (item, type, keys[item]))
+
+def write_up_bootsector(device, partition):
+    """Write out the bootsector to a utility partition"""
+    #build the bootsector of the partition
+    if os.path.exists('/usr/share/dell/up/up.bs'):
+        with open('/usr/share/dell/up/up.bs', 'rb') as rfd:
+            with open(device + partition, 'wb') as wfd:
+                wfd.write(rfd.read(11))  # writes the jump to instruction and oem name
+                rfd.seek(43)
+                wfd.seek(43)
+                wfd.write(rfd.read(469)) # write the label, FS type, bootstrap code and signature
+            #If we don't have the bootsector code, then just set the label properly
+    else:
+        fetch_output(['dosfslabel', device + partition, "DellUtility"])
+
+
 def dbus_sync_call_signal_wrapper(dbus_iface, func, handler_map, *args, **kwargs):
     '''Run a D-BUS method call while receiving signals.
 
@@ -600,10 +734,10 @@ def dbus_sync_call_signal_wrapper(dbus_iface, func, handler_map, *args, **kwargs
         # not a D-BUS object
         return getattr(dbus_iface, func)(*args, **kwargs)
 
-    def _h_reply(result=None):
+    def _h_reply(*args, **kwargs):
         """protected method to send a reply"""
         global _h_reply_result
-        _h_reply_result = result
+        _h_reply_result = args
         loop.quit()
 
     def _h_error(exception=None):
